@@ -15,8 +15,25 @@
 #'
 TransformSigma_PDtoR3 <- function(vec){
   vec[2] <- atanh(vec[2] / (sqrt(vec[1]) * sqrt(vec[3])))
+  vec[c(1,3)] = TransformSigma_PDtoR2(vec[c(1,3)])
+  return(vec)
+}
+
+#' TransformSigma_PDtoR2
+#'
+#' @param vec A length 2 vector of the variance of equation 1 followed by the variance of equation 2.
+#'
+#' @return A length 2 vector spanning unrestricted R2.
+#'
+#' @export
+#'
+#' @examples
+#'PD_vec <- c(1, 1)
+#'TransformSigma_PDtoR2(PD_vec)
+#'
+TransformSigma_PDtoR2 <- function(vec){
   vec[1] <- log(vec[1])
-  vec[3] <- log(vec[3])
+  vec[2] <- log(vec[2])
   return(vec)
 }
 
@@ -35,12 +52,140 @@ TransformSigma_PDtoR3 <- function(vec){
 #'TransformSigma_R3toPD(R3_vec)
 #'
 TransformSigma_R3toPD <- function(vec){
-  vec[1] <- exp(vec[1])
-  vec[3] <- exp(vec[3])
+  vec[c(1,3)] = TransformSigma_R2toPD(vec[c(1,3)])
   vec[2] <- tanh(vec[2]) * sqrt(vec[1]) * sqrt(vec[3])
   return(vec)
 }
 
+#' TransformSigma_R2toPD
+#'
+#' @param vec A length 2 vector output from TransformSigma_PDtoR2.
+#'
+#' @return A length 2 vector of variances spanning a positive definite space.
+#'
+#' @export
+#'
+#' @examples
+#'PD_vec <- c(1, 1)
+#'R2_vec <- TransformSigma_PDtoR2(PD_vec)
+#'TransformSigma_R2toPD(R2_vec)
+#'
+TransformSigma_R2toPD <- function(vec){
+  vec[1] <- exp(vec[1])
+  vec[2] <- exp(vec[2])
+  return(vec)
+}
+
+#' GetDeltaMethodParameters
+#'
+#' @description
+#'
+#' Transforms the mean and variance covariance matrix of the estimators in an unrestricted space to a positive definite space for the delta method.
+#'
+#' @param mu A numeric vector output from DE()$par
+#'
+#' @param covmat A numeric matrix output from DE()$vcov
+#'
+#' @param MaskRho The output from DE()$MaskRho
+#'
+#' @return A list containing the parameters of the normal distribution after transforming the coavariance variables to a positive definite space
+#'
+#' @export
+#'
+#' @examples
+#'
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
+#'
+#'results = DE(formula = Y ~ X1 | X2, data = df)
+#'
+#'GetDeltaMethodParameters(results$par,results$vcov,results$MaskRho)
+#'
+GetDeltaMethodParameters <- function(mu,covmat = NULL,MaskRho = FALSE){
+
+  pall = length(mu)
+  if(isFALSE(MaskRho)){
+    p = pall - 3
+  }else{
+    p = pall - 2
+  }
+
+  transmu = mu
+  if(isFALSE(MaskRho)){
+    transmu[p + 1:3] = TransformSigma_R3toPD(mu[p + 1:3])
+  }else{
+    transmu[p + 1:2] = TransformSigma_R2toPD(mu[p + 1:2])
+  }
+
+  if(!is.null(names(mu))){
+    names(transmu) = names(mu)
+    if(isFALSE(MaskRho)){
+      names(transmu)[length(transmu)- c(2,0)] = gsub("log","",names(transmu)[length(transmu)- c(2,0)])
+      names(transmu)[length(transmu)- 1] = "Covariance"
+    }else{
+      names(transmu)[length(transmu)- c(1,0)] = gsub("log","",names(transmu)[length(transmu)- c(1,0)])
+    }
+  }
+
+
+  out = list(mu = transmu)
+
+  # Multivariate to Multivariate delta method, Poirier(1995) Theorem 5.7.7
+  if(!is.null(covmat)){
+    dgdtheta = matrix(0,nrow = pall, ncol = pall)
+    dgdtheta[cbind(1:p,1:p)] = 1
+    dgdtheta[p+1,p+1] = exp(mu[p+1])
+
+    if(isFALSE(MaskRho)){
+      dgdtheta[p+2,p + 1:3] = exp(sum(mu[p+c(1,3)])) *
+        c(
+         tanh(mu[p+2]),
+        (cosh(mu[p+2]) ^ (-2)),
+         tanh(mu[p+2])
+        )
+    }
+
+    dgdtheta[pall,pall] = exp(mu[pall])
+
+    transcovmat =  dgdtheta %*% covmat %*% t(dgdtheta)
+    out$covmat = transcovmat
+
+    if(!is.null(dimnames(covmat))){
+      dimnames(out$covmat) = dimnames(covmat)
+      if(isFALSE(MaskRho)){
+        colnames(out$covmat)[nrow(out$covmat)- c(2,0)] = gsub("log","",colnames(out$covmat)[nrow(out$covmat)- c(2,0)])
+        colnames(out$covmat)[nrow(out$covmat)- 1] = "Covariance"
+      }else{
+        colnames(out$covmat)[nrow(out$covmat)- c(1,0)] = gsub("log","",colnames(out$covmat)[nrow(out$covmat)- c(1,0)])
+      }
+      rownames(out$covmat) = colnames(out$covmat)
+    }
+
+  }
+
+  return(out)
+
+}
+
 #' Derivative of likelihood with respect to the inverse hyperbolic tangent of correlation
 #'
 #' @param Y A vector of observed responses.
@@ -131,7 +276,7 @@ DlhoodDatanhrho <- function(Y,mu,logsigma11,logsigma22,atanhrho){
 #'p2 = 2
 #'theta = c(beta01, beta02, log(SigmaX[1, 1]), atanh(SigmaX[1, 2]), log(SigmaX[2, 2]))
 #'mu = cbind(X[[1]] %*% theta[1:p1], X[[2]] %*% theta[(p1 + 1):(p1 + p2)])
-#'lhood = exp(-LLikelihoodDE(theta, Y, X, ensurePD = TRUE, summed = FALSE))
+#'lhood = exp(-nLLikelihoodDE(theta, Y, X, transformR3toPD = TRUE, summed = FALSE))
 #'
 #'d <- DllhoodDatanhrho(Y = Y, mu = mu, logsigma11 = theta[p1 + p2 + 1],
 #'    logsigma22 = theta[p1 + p2 + 3], atanhrho = theta[p1 + p2 + 2], lhood = lhood)
@@ -235,7 +380,7 @@ DlhoodDlogsigma11 <- function(Y,mu,logsigma11,logsigma22,atanhrho){
 #'p2 = 2
 #'theta = c(beta01, beta02, log(SigmaX[1, 1]), atanh(SigmaX[1, 2]), log(SigmaX[2, 2]))
 #'mu = cbind(X[[1]] %*% theta[1:p1], X[[2]] %*% theta[(p1 + 1):(p1 + p2)])
-#'lhood = exp(-LLikelihoodDE(theta, Y, X, ensurePD = TRUE, summed = FALSE))
+#'lhood = exp(-nLLikelihoodDE(theta, Y, X, transformR3toPD = TRUE, summed = FALSE))
 #'
 #'d <- DllhoodDlogsigma11(Y = Y, mu = mu, logsigma11 = theta[p1 + p2 + 1],
 #'    logsigma22 = theta[p1 + p2 + 3], atanhrho = theta[p1 + p2 + 2], lhood = lhood)
@@ -347,7 +492,7 @@ DlhoodDbeta1 <- function(Y,mu,logsigma11,logsigma22,atanhrho,X1){
 #'p2 = 2
 #'theta = c(beta01, beta02, log(SigmaX[1, 1]), atanh(SigmaX[1, 2]), log(SigmaX[2, 2]))
 #'mu = cbind(X[[1]] %*% theta[1:p1], X[[2]] %*% theta[(p1 + 1):(p1 + p2)])
-#'lhood = exp(-LLikelihoodDE(theta, Y, X, ensurePD = TRUE, summed = FALSE))
+#'lhood = exp(-nLLikelihoodDE(theta, Y, X, transformR3toPD = TRUE, summed = FALSE))
 #'
 #'d = DllhoodDbeta1(Y = Y, mu = mu, logsigma11 = theta[p1 + p2 + 1],
 #'    logsigma22 = theta[p1 + p2 + 3], atanhrho = theta[p1 + p2 + 2], lhood = lhood, X1 = X1)
@@ -370,6 +515,9 @@ DllhoodDbeta1 <- function(Y,mu,logsigma11,logsigma22,atanhrho,lhood,X1){
 #' design matrix for equation 1 and the second element is a \eqn{N \times k_2}{N x k[2]}
 #' design matrix for equation 2.
 #' @param summed A logical to determine if gradient values are summed over observations.
+#' @param MaskRho A logical or numeric to determine if the correlation is masked.
+#' A value of FALSE means the correlation is not fixed. A value between -1 and 1 will fix the
+#' correlation to that value.
 #'
 #' @return A \eqn{(k_1 + k_2 + 3)}{(k[1] + k[2] + 3)} dimension vector of
 #' derivatives if \code{summed = TRUE}, else a
@@ -405,27 +553,41 @@ DllhoodDbeta1 <- function(Y,mu,logsigma11,logsigma22,atanhrho,lhood,X1){
 #'Gradient = GradientDE(theta, Y, X, summed = TRUE)
 #'head(Gradient)
 #'
-GradientDE <- function(theta, Y, X, summed = TRUE){
+GradientDE <- function(theta, Y, X, summed = TRUE, MaskRho = FALSE){
 
   p1 = ncol(X[[1]])
   p2 = ncol(X[[2]])
   p = p1 + p2
 
   # Set up parameters
-  mu = cbind(X[[1]] %*% theta[1:p1], X[[2]] %*% theta[(p1 + 1):(p1 + p2)])
+  mu = cbind(X[[1]] %*% theta[1:p1], X[[2]] %*% theta[(p1 + 1):p])
   logsigma11 = theta[p+1]
-  atanhrho = theta[p+2]
-  logsigma22 = theta[p+3]
+  if(isFALSE(MaskRho)){
+    atanhrho = theta[p+2]
+    logsigma22 = theta[p+3]
+  }else{
+    atanhrho = atanh(MaskRho)
+    logsigma22 = theta[p+2]
+  }
 
   # Precompute likelihood
-  lhood = exp(-LLikelihoodDE(theta, Y, X, ensurePD = TRUE, summed = FALSE))
+  lhood = exp(-nLLikelihoodDE(theta, Y, X, transformR3toPD = TRUE, summed = FALSE, MaskRho = MaskRho))
+
 
   gradvec = cbind(
     DllhoodDbeta1(Y,mu,logsigma11,logsigma22,atanhrho,lhood,X[[1]]),
     DllhoodDbeta1(Y,mu[,2:1],logsigma22,logsigma11,atanhrho,lhood,X[[2]]),
-    DllhoodDlogsigma11(Y,mu,logsigma11,logsigma22,atanhrho,lhood),
-    DllhoodDatanhrho(Y,mu,logsigma11,logsigma22,atanhrho,lhood),
-    DllhoodDlogsigma11(Y,mu[,2:1],logsigma22,logsigma11,atanhrho,lhood))
+    DllhoodDlogsigma11(Y,mu,logsigma11,logsigma22,atanhrho,lhood))
+  if(isFALSE(MaskRho)){
+    gradvec = cbind(gradvec,
+                    DllhoodDatanhrho(Y,mu,logsigma11,logsigma22,atanhrho,lhood))
+  }
+  gradvec = cbind(gradvec,
+                  DllhoodDlogsigma11(Y,mu[,2:1],logsigma22,logsigma11,atanhrho,lhood))
+
+  if(!is.null(names(theta))){
+    colnames(gradvec) = names(theta)
+  }
 
   if(summed){
     return(colSums(gradvec))
@@ -447,6 +609,9 @@ GradientDE <- function(theta, Y, X, summed = TRUE){
 #' design matrix for equation 1 and the second element is a \eqn{N \times k_2}{N x k[2]}
 #' design matrix for equation 2.
 #' @param summed A logical to determine if gradient values are summed over observations.
+#' @param MaskRho A logical or numeric to determine if the correlation is masked.
+#' A value of FALSE means the correlation is not fixed. A value between -1 and 1 will fix the
+#' correlation to that value.
 #'
 #' @return A \eqn{(k_1 + k_2 + 3)}{(k[1] + k[2] + 3)} dimension vector of
 #' derivatives if \code{summed = TRUE}, else a
@@ -483,13 +648,55 @@ GradientDE <- function(theta, Y, X, summed = TRUE){
 #'Gradient = nGradientDE(theta, Y, X, summed = TRUE)
 #'head(Gradient)
 #'
-nGradientDE <- function(theta, Y, X, summed = TRUE){
-  return(-GradientDE(theta, Y, X, summed = summed))
+nGradientDE <- function(theta, Y, X, summed = TRUE, MaskRho = FALSE){
+  return(-GradientDE(theta, Y, X, summed = summed, MaskRho = MaskRho))
 }
 
 
 #' Negative log likelihood of market in disequilibrium model
 #'
+#' @description A wrapper function that makes the output of LLikelihoodDE negative.
+#' See ?LLikelihoodDE for details.
+#'
+#' @param ... arguements to be passed to LLikelhoodDE
+#' @export
+#'
+#' @examples
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'
+#'p1 = 2
+#'p2 = 2
+#'theta = c(beta01, beta02, log(SigmaX[1, 1]), atanh(SigmaX[1, 2]), log(SigmaX[2, 2]))
+#'
+#'lhood = nLLikelihoodDE(theta, Y, X, summed = TRUE)
+#'head(nLLikelihoodDE)
+#'
+nLLikelihoodDE <- function(...){
+  return(-LLikelihoodDE(...))
+}
+
+
+#' Log likelihood of market in disequilibrium model
+#'
 #' @param theta A vector of parameter values to obtain the gradient at. The
 #' order of parameters is coefficients of equation 1, coefficients of equation 2,
 #' variance of equation 1, correlation of equations 1 and 2,
@@ -498,11 +705,14 @@ nGradientDE <- function(theta, Y, X, summed = TRUE){
 #' @param X A list of two elements. The first element is a \eqn{N \times k_1}{N x k[1]}
 #' design matrix for equation 1 and the second element is a \eqn{N \times k_2}{N x k[2]}
 #' design matrix for equation 2.
-#' @param ensurePD A logical  to determine if the covariance matrix is
-#' transformed to an unrestricted 3 dimension real space (\code{ensurePD = TRUE})
-#' or not (\code{ensurePD = FALSE}).
+#' @param transformR3toPD A logical  to determine if the covariance matrix is
+#' transformed to an unrestricted 3 dimension real space (\code{transformR3toPD = TRUE})
+#' or not (\code{transformR3toPD = FALSE}).
 #' @param summed A logical to determine if the negative log likelihood values
 #' are summed over observations.
+#' @param MaskRho A logical or numeric to determine if the correlation is masked.
+#' A value of FALSE means the correlation is not fixed. A value between -1 and 1 will fix the
+#' correlation to that value.
 #'
 #' @return A scalar value of the negative log likelihood if \code{summed = TRUE},
 #' else a \eqn{N}{N} length vector of negative log likelihood observations.
@@ -537,20 +747,33 @@ nGradientDE <- function(theta, Y, X, summed = TRUE){
 #'lhood = LLikelihoodDE(theta, Y, X, summed = TRUE)
 #'head(LLikelihoodDE)
 #'
-LLikelihoodDE <- function(theta, Y, X, ensurePD = TRUE, summed = TRUE){
+LLikelihoodDE <- function(theta, Y, X, transformR3toPD = TRUE, summed = TRUE, MaskRho = FALSE){
   p1 <- ncol(X[[1]])
   p2 <- ncol(X[[2]])
   p <- p1 + p2
   beta <- list(beta1 = theta[1:p1], beta2 = theta[(p1 + 1):p])
-  Sigma <- matrix(c(theta[p + 1], theta[p + 2], theta[p + 2], theta[p + 3]), 2, 2)
-
-  # convert covariance matrix to normal parameter space
-  if(ensurePD){
-    newSigma <- TransformSigma_R3toPD(c(Sigma[1, 1], Sigma[1, 2], Sigma[2, 2]))
-    Sigma[1, 1] <- newSigma[1]
-    Sigma[1, 2] <- newSigma[2]
-    Sigma[2, 1] <- Sigma[1, 2]
-    Sigma[2, 2] <- newSigma[3]
+  if(isFALSE(MaskRho)){
+    Sigma <- matrix(c(theta[p + 1], theta[p + 2],
+                      theta[p + 2], theta[p + 3]), 2, 2)
+    if(transformR3toPD){
+      newSigma <- TransformSigma_R3toPD(c(Sigma[1, 1], Sigma[1, 2], Sigma[2, 2]))
+      Sigma[1, 1] <- newSigma[1]
+      Sigma[1, 2] <- newSigma[2]
+      Sigma[2, 1] <- Sigma[1, 2]
+      Sigma[2, 2] <- newSigma[3]
+    }
+  }else{
+    if(transformR3toPD){
+      Sigma <- matrix(NA, 2, 2)
+      newSigma <- TransformSigma_R2toPD(theta[p+1:2])
+      Sigma[1, 1] <- newSigma[1]
+      Sigma[2, 2] <- newSigma[2]
+      Sigma[1, 2] <- MaskRho * sqrt(Sigma[1, 1]) * sqrt(Sigma[2, 2])
+      Sigma[2, 1] <- Sigma[1, 2]
+    }else{
+      Sigma <- matrix(c(theta[p + 1], MaskRho * sqrt(theta[p + 1]) * sqrt(theta[p + 2]),
+                        MaskRho * sqrt(theta[p + 1]) * sqrt(theta[p + 2]), theta[p + 2]), 2, 2)
+    }
   }
 
   # precomputation
@@ -572,13 +795,13 @@ LLikelihoodDE <- function(theta, Y, X, ensurePD = TRUE, summed = TRUE){
 
   llhood <- log(PrY)
   if(summed){
-    return(-sum(llhood))
+    out = sum(llhood)
   } else {
-    return(-llhood)
+    out = llhood
   }
+  return(out)
 }
 
-
 #' @title Market in Disequilibrium Model
 #'
 #' @description
@@ -647,9 +870,43 @@ LLikelihoodDE <- function(theta, Y, X, ensurePD = TRUE, summed = TRUE){
 #'
 #' \item{Xbeta2}{The predicted outcome for each observation in equation 2 using the parameters estimated in \code{par}.}
 #'
+#' \item{Y}{A vector of the response (quantity) values.}
+#'
+#' \item{X}{A list of the design matricies for the two equations.}
+#'
 #' \item{hessian}{A numerical approximation to the Hessian matrix of the likelihood function at the estimated parameter values.
 #'  The Hessian is returned even if it is not requested.}
 #'
+#' \item{vcov}{The variance covariance matrix of the estimates in \code{par}.}
+#'
+#' \item{MaskRho}{The value of \code{MaskRho} used.}
+#'
+#' }
+#'
+#' The object of class 'DE' has the following attributes:
+#'
+#' \describe{
+#'
+#' \item{originalNamesX1}{The original names of the covariates for equation 1 specified in \code{formula}.}
+#'
+#' \item{originalNamesX2}{The original names of the covariates for equation 2 specified in \code{formula}.}
+#'
+#' \item{namesX1}{The names of the covariates for equation 1 to be used in the \code{summary.DE} function.
+#'  This is equivalent to \code{paste0(originalNamesX1, Equation1Name)}.}
+#'
+#' \item{namesX2}{The names of the covariates for equation 2 to be used in the \code{summary.DE} function.
+#'  This is equivalent to \code{paste0(originalNamesX2, Equation2Name)}.}
+#'
+#' \item{namesSigma}{The names of the variance-covariance matrix parameters to be used in the \code{summary.DE} function.}
+#'
+#' \item{Equation1Name}{The user-specified name of equation 1.}
+#'
+#' \item{Equation2Name}{The user-specified name of equation 2.}
+#'
+#' \item{betaN1}{The number of coefficient and slope parameters to be estimated in equation 1.}
+#'
+#' \item{betaN2}{The number of coefficient and slope parameters to be estimated in equation 2.}
+#'
 #' }
 #'
 #'
@@ -684,46 +941,25 @@ LLikelihoodDE <- function(theta, Y, X, ensurePD = TRUE, summed = TRUE){
 #' \item{na.action}{A function indicating what happens when data contains NAs. The default is \link{na.omit}.
 #'  The only other possible value is \link{na.fail}.}
 #'
-#' \item{ensurePD}{A logical. If TRUE, the covariance matrix will be manually converted to a
+#' \item{transformR3toPD}{A logical. If TRUE, the covariance matrix will be manually converted to a
 #'  positive definite matrix for optimization. The default is TRUE.}
 #'
 #' \item{Equation1Name}{A string name for the first equation. The default is "_1".}
 #'
 #' \item{Equation2Name}{A string name for the second equation. The default is "_2".}
 #'
+#' \item{MaskRho}{A logical or numeric to determine if the correlation is masked.
+#' A value of FALSE means the correlation is not fixed. A value between -1 and 1 will fix the
+#' correlation to that value. The default is FALSE. A free correlation parameter can be numerically unstable, use with caution.}
+#'
 #'}
 #'
-#' An object of class 'DE' is returned as a list with the following attributes:
-#'
-#' \describe{
-#'
-#' \item{originalNamesX1}{The original names of the covariates for equation 1 specified in \code{formula}.}
-#'
-#' \item{originalNamesX2}{The original names of the covariates for equation 2 specified in \code{formula}.}
-#'
-#' \item{namesX1}{The names of the covariates for equation 1 to be used in the \code{summary.DE} function.
-#'  This is equivalent to \code{paste0(originalNamesX1, Equation1Name)}.}
-#'
-#' \item{namesX2}{The names of the covariates for equation 2 to be used in the \code{summary.DE} function.
-#'  This is equivalent to \code{paste0(originalNamesX2, Equation2Name)}.}
-#'
-#' \item{namesSigma}{The names of the variance-covariance matrix parameters to be used in the \code{summary.DE} function.}
-#'
-#' \item{Equation1Name}{The user-specified name of equation 1.}
-#'
-#' \item{Equation2Name}{The user-specified name of equation 2.}
-#'
-#' \item{betaN1}{The number of coefficient and slope parameters to be estimated in equation 1.}
-#'
-#' \item{betaN2}{The number of coefficient and slope parameters to be estimated in equation 2.}
-#'
-#' }
 #'
 #'
 #' @references
-#' \cite{Gourieroux, C. (2000). Econometrics of Qualitative Dependent Variables (Themes in Modern Econometrics) (P. Klassen, Trans.). Cambridge: Cambridge University Press. doi:10.1017/CBO9780511805608}
+#' \cite{Gourieroux, C. (2000). Econometrics of Qualitative Dependent Variables (Themes in Modern Econometrics) (P. Klassen, Trans.). Cambridge: Cambridge University Press. http://doi.org/10.1017/CBO9780511805608}
 #'
-#' \cite{Maddala, G. (1983). Limited-Dependent and Qualitative Variables in Econometrics (Econometric Society Monographs). Cambridge: Cambridge University Press. doi:10.1017/CBO9780511810176}
+#' \cite{Maddala, G. (1983). Limited-Dependent and Qualitative Variables in Econometrics (Econometric Society Monographs). Cambridge: Cambridge University Press. http://doi.org/10.1017/CBO9780511810176}
 #'
 #' @examples
 #'set.seed(1775)
@@ -746,7 +982,7 @@ LLikelihoodDE <- function(theta, Y, X, ensurePD = TRUE, summed = TRUE){
 #'Y1 = X1 %*% beta01 + eps1
 #'Y2 = X2 %*% beta02 + eps2
 #'Y = pmin(Y1,Y2)
-#'df = data.frame(Y = Y, X1 = X1, X2 = X2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
 #'
 #'results = DE(formula = Y ~ X1 | X2, data = df)
 #'
@@ -800,19 +1036,42 @@ DE <- function(formula, data, subset = NULL, par = NULL, control = list()){
   if(!is.numeric(c(Y, X1, X2))){
     stop('variables in formula must be numeric')
   }
-  if(!is.null(par)){
-    if(length(par) != p + 3){
-      stop(paste0('par must be a vector of length equal to the number of free parameters (', p + 3, ')'))
+  if(is.null(control$MaskRho)){
+    control$MaskRho = FALSE
+  }else if(isTRUE(control$MaskRho)){
+    control$MaskRho = 0
+  }else if(is.numeric(control$MaskRho)){
+    if(abs(control$MaskRho) > 1){
+      stop("control$MaskRho must be unspecified, logical, or a numeric between -1 and 1")
     }
   }
+  if(!is.null(par)){
+    if(isFALSE(control$MaskRho)){
+      if(length(par) != p + 3){
+        stop(paste0('par must be a vector of length equal to the number of free parameters (', p + 3, ')'))
+      }
+    }else{
+      if(length(par) != p + 2){
+        stop(paste0('par must be a vector of length equal to the number of free parameters (', p + 2, ')'))
+      }
+    }
+  }
+
   if(!is.null(par) & !is.numeric(par)){
     stop('par must be a numeric vector')
   }
   if(!is.null(par)){
-    if((par[p + 1] <= 0) |
-       (par[p + 3] <= 0) |
-       (par[p + 3] - par[p + 2] / par[p + 1] < 0)){
-      stop('variance-covariance parameters in par must correspond to a positive definite matrix')
+    if(isFALSE(control$MaskRho)){
+      if((par[p + 1] <= 0) |
+         (par[p + 3] <= 0) |
+         (par[p + 3] - par[p + 2] / par[p + 1] < 0)){
+        stop('variance-covariance parameters in par must correspond to a positive definite matrix')
+      }
+    }else{
+      if((par[p + 1] <= 0) |
+         (par[p + 2] <= 0)){
+        stop('variance-covariance parameters in par must correspond to a positive definite matrix')
+      }
     }
   }
   if(!is.numeric(control$lower)){
@@ -834,18 +1093,32 @@ DE <- function(formula, data, subset = NULL, par = NULL, control = list()){
     control$Equation2Name <- '_2'
   }
 
-  # prep data
+  # prep parameters
   if(is.null(par)){
-    par <- c(rep(0, p), 1, 0, 1)
+    getintercept1 = which(apply(X1,2,function(x){length(unique(x))})==1)
+    getintercept2 = which(apply(X2,2,function(x){length(unique(x))})==1)
+    par <- c(rep(0, p), stats::var(Y), 0, stats::var(Y))
+    par[getintercept1] = mean(Y)
+    par[p1 + getintercept2] = mean(Y)
+    if(isFALSE(control$MaskRho)){
+      par[(p + 1):(p + 3)] <- TransformSigma_PDtoR3(par[(p + 1):(p + 3)])
+    }else{
+      par = par[-(p+2)]
+    }
+  }else{
+    par[(p + 1):(p + 2)] <- TransformSigma_PDtoR2(par[(p + 1):(p + 2)])
   }
-  par[(p + 1):(p + 3)] <- TransformSigma_PDtoR3(par[(p + 1):(p + 3)])
 
   # MLE
-  out <- optimr::optimr(par = par, fn = LLikelihoodDE, gr = nGradientDE, lower = control$lower,
+  out <- optimr::optimr(par = par, fn = nLLikelihoodDE, gr = nGradientDE, lower = control$lower,
                         upper = control$upper,
                         method = control$method, hessian = control$hessian,
-                        X = list(X1, X2), Y = Y, control = control)
-  out$par[(p + 1):(p + 3)] <- TransformSigma_R3toPD(out$par[(p + 1):(p + 3)])
+                        X = list(X1, X2), Y = Y, MaskRho = control$MaskRho, control = control)
+  #out$par[(p + 1):(p + 3)] <- TransformSigma_R3toPD(out$par[(p + 1):(p + 3)])
+
+  if(out$convergence !=0){
+    warning("The optimization did not converge.")
+  }
 
   # create DE class
   class(out) <- c(class(out), 'DE')
@@ -853,26 +1126,54 @@ DE <- function(formula, data, subset = NULL, par = NULL, control = list()){
   attr(out, 'originalNamesX2') <- originalNamesX2
   attr(out, 'namesX1') <- paste0(originalNamesX1, control$Equation1Name)
   attr(out, 'namesX2') <- paste0(originalNamesX2, control$Equation2Name)
-  attr(out, 'namesSigma') <- c(paste0('Variance', control$Equation1Name), 'Covariance', paste0('Variance', control$Equation2Name))
+  if(isFALSE(control$MaskRho)){
+    attr(out, 'namesSigma') <- c(paste0('Variance', control$Equation1Name), 'Covariance', paste0('Variance', control$Equation2Name))
+  }else{
+    attr(out, 'namesSigma') <- c(paste0('Variance', control$Equation1Name), paste0('Variance', control$Equation2Name))
+  }
+
+  if(isFALSE(control$MaskRho)){
+    names(out$par) = c( paste0(originalNamesX1, control$Equation1Name),
+                        paste0(originalNamesX2, control$Equation2Name),
+                        paste0('logVariance', control$Equation1Name), 'atanhRho', paste0('logVariance', control$Equation2Name))
+  }else{
+    names(out$par) = c( paste0(originalNamesX1, control$Equation1Name),
+                        paste0(originalNamesX2, control$Equation2Name),
+                        paste0('logVariance', control$Equation1Name), paste0('logVariance', control$Equation2Name))
+  }
   attr(out, 'Equation1Name') <- control$Equation1Name
   attr(out, 'Equation2Name') <- control$Equation2Name
   attr(out, 'betaN1') <- p1
   attr(out, 'betaN2') <- p2
-  out$Sigma <- matrix(c(out$par[p + 1], out$par[p + 2], out$par[p + 2], out$par[p + 3]), nrow = 2, ncol = 2)
+  if(isFALSE(control$MaskRho)){
+    temp <- TransformSigma_R3toPD(out$par[(p + 1):(p + 3)])
+    out$Sigma = matrix(temp[c(1,2,2,3)],2,2)
+  }else{
+    temp <- TransformSigma_R2toPD(out$par[(p + 1):(p + 2)])
+    out$Sigma = matrix(c(temp[1],control$MaskRho,control$MaskRho,temp[2]),2,2)
+  }
   Xbeta1 <- X1 %*% matrix(out$par[1:p1], nrow = p1, ncol = 1)
   attributes(Xbeta1)$dimnames <- NULL
   out$Xbeta1 <- Xbeta1
   Xbeta2 <- X2 %*% matrix(out$par[(p1 + 1):(p1 + p2)], nrow = p2, ncol = 1)
   attributes(Xbeta2)$dimnames <- NULL
   out$Xbeta2 <- Xbeta2
-  out$hessian <- numDeriv::hessian(func = LLikelihoodDE, x = out$par,
-                                   X = list(X1, X2), Y = Y, ensurePD = FALSE)
+  out$Y = Y
+  out$X = list(X1, X2)
+  names(out$X) = c(control$Equation1Name, control$Equation2Name)
+  out$hessian <- numDeriv::hessian(func = nLLikelihoodDE, x = out$par,
+                                   X = list(X1, X2), Y = Y, MaskRho = control$MaskRho, transformR3toPD = TRUE)
+  dimnames(out$hessian)  = list(names(out$par),names(out$par))
+  out$vcov = tryCatch(expr = chol2inv(chol(out$hessian)), error = function(x){matrix(NA,length(out$par),length(out$par))})
+  dimnames(out$vcov) = list(names(out$par),names(out$par))
+  out$MaskRho = control$MaskRho
   return(out)
 }
 
 #' @title Summary method for class 'DE'
 #'
 #' @param object An object of class \code{DE} for which a summary is desired.
+#' @param robust A logical determining if robust standard errors should be used. If true standard errors are from sandwich::sandwich, else hessian.
 #' @param ... Unused
 #'
 #' @return
@@ -914,25 +1215,42 @@ DE <- function(formula, data, subset = NULL, par = NULL, control = list()){
 #'Y1 = X1 %*% beta01 + eps1
 #'Y2 = X2 %*% beta02 + eps2
 #'Y = pmin(Y1,Y2)
-#'df = data.frame(Y = Y, X1 = X1, X2 = X2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
 #'
 #'results = DE(formula = Y ~ X1 | X2, data = df)
 #'
 #'summary(results)
 #'
-summary.DE <- function(object,...){
+summary.DE <- function(object,robust = FALSE,...){
   dots <- match.call(expand.dots = TRUE)
   if(is.null(dots$digits)){
     dots$digits <- max(3, getOption("digits") - 3)
   }
-  coeff <- object$par
-  if(!is.null(object$hessian)){
-    se <- sqrt(diag(solve(object$hessian)))
-  } else {
+
+  if(!any(is.na(object$vcov))){
+    deltamethpar = GetDeltaMethodParameters(object$par,object$vcov)
+    transcovmat = deltamethpar$covmat
+
+    if(isFALSE(robust)){
+      transcovmat = sandwich::sandwich(object)
+    }
+
+
+    se <- sqrt(diag(transcovmat))
+  }else{
+    deltamethpar = GetDeltaMethodParameters(object$par)
     se <- rep(NA,length(coeff))
   }
+
+
+  coeff = deltamethpar$mu
   zstat <- abs(coeff / se)
   pval <- stats::pnorm(zstat, lower.tail = FALSE) * 2
+  if(isFALSE(object$MaskRho)){
+    pval[length(pval) - c(0,2)] = NA
+  }else{
+    pval[length(pval) - c(0,1)] = NA
+  }
   out <- cbind(coeff, se, zstat, pval)
   colnames(out) <- c('Estimate', 'Std. Error', 'z value', 'Pr(>|z|)')
   rownames(out) <- c(attr(object, 'namesX1'), attr(object, 'namesX2'), attr(object, 'namesSigma'))
@@ -959,7 +1277,7 @@ summary.DE <- function(object,...){
 #' \item{Min(Y_1,Y_2)}{The minimum of \code{Y_1} and \code{Y_2}.}
 #'
 #' \item{Prob(Y_2>Y_1)}{The probability that the outcome variable in equation 2 is greater than the outcome
-#' variable in equation 1.  This is the probability that \code{Y_1} is the observed quantity.}
+#' variable in equation 1.  This is the probability that \code{Y_1} is the observed quantity. This probability does not account for estimation uncertainty. Also note that all predictions are unconditional on the observed quantity.}
 #'
 #' }
 #'
@@ -986,7 +1304,7 @@ summary.DE <- function(object,...){
 #'Y1 = X1 %*% beta01 + eps1
 #'Y2 = X2 %*% beta02 + eps2
 #'Y = pmin(Y1,Y2)
-#'df = data.frame(Y = Y, X1 = X1, X2 = X2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
 #'
 #'results = DE(formula = Y ~ X1 | X2, data = df)
 #'
@@ -1019,4 +1337,206 @@ predict.DE <- function(object, newdata = NULL,...){
   return(out)
 }
 
+#' @title estfun method for class 'DE'
+#'
+#' @param x An object of class \code{DE}.
+#' @param ... Unused
+#'
+#' @return
+#' A \eqn{N \times (k_1 + k_2 + 3)}{N x (k[1] + k[2] + 3)} if rho is not masked (otherwise \eqn{N \times (k_1 + k_2 + 2)}{N x (k[1] + k[2] + 2)}) matrix of the gradient is returned.
+#'
+#' @export
+#'
+#'
+#'
+#' @examples
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
+#'
+#'results = DE(formula = Y ~ X1 | X2, data = df)
+#'
+#'head(sandwich::estfun(results))
+#'
+estfun.DE <- function(x, ...){
+
+  out =  GradientDE(theta = x$par,Y = x$Y, X = x$X , summed = FALSE, MaskRho = x$MaskRho)
+
+ return(out)
+
+}
+
+#' @title vcov method for class 'DE'
+#'
+#' @param object An object of class \code{DE}.
+#' @param ... Unused
+#'
+#' @return
+#' A \eqn{N \times (k_1 + k_2 + 3)}{N x (k[1] + k[2] + 3)} if matrix of the gradient is returned.
+#'
+#' @export
+#'
+#'
+#'
+#' @examples
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
+#'
+#'results = DE(formula = Y ~ X1 | X2, data = df)
+#'
+#'vcov(results)
+#'
+vcov.DE <- function(object, ...){
+
+  out =  object$vcov
+
+  return(out)
+
+}
+
+#' @title nobs method for class 'DE'
+#'
+#' @param object An object of class \code{DE}.
+#' @param ... Unused
+#'
+#' @return
+#' A scalar representing the number of observations \eqn{N}{N}.
+#'
+#' @export
+#'
+#'
+#'
+#' @examples
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
+#'
+#'results = DE(formula = Y ~ X1 | X2, data = df)
+#'
+#'nobs(results)
+#'
+nobs.DE <- function(object, ...){
+
+  out =  length(object$Y)
+
+  return(out)
+
+}
+
+#' @title residuals method for class 'DE'
+#'
+#' @param object An object of class \code{DE}.
+#' @param ... Unused
+#'
+#' @return
+#' A 2 dimension matrix of raw residuals for equations 1 and 2. Allows use with Sandwich package.
+#'
+#' @export
+#'
+#'
+#' @examples
+#'set.seed(1775)
+#'library(MASS)
+#'beta01 = c(1,1)
+#'beta02 = c(-1,-1)
+#'N = 10000
+#'SigmaEps = diag(2)
+#'SigmaX = diag(2)
+#'MuX = c(0,0)
+#'par0 = c(beta01, beta02, SigmaX[1, 1], SigmaX[1, 2], SigmaX[2, 2])
+#'
+#'Xgen = mvrnorm(N,MuX,SigmaX)
+#'X1 = cbind(1,Xgen[,1])
+#'X2 = cbind(1,Xgen[,2])
+#'X = list(X1 = X1,X2 = X2)
+#'eps = mvrnorm(N,c(0,0),SigmaEps)
+#'eps1 = eps[,1]
+#'eps2 = eps[,2]
+#'Y1 = X1 %*% beta01 + eps1
+#'Y2 = X2 %*% beta02 + eps2
+#'Y = pmin(Y1,Y2)
+#'df = data.frame(Y = Y, X1 = Xgen[,1], X2 = Xgen[,2])
+#'
+#'results = DE(formula = Y ~ X1 | X2, data = df)
+#'
+#'head(residuals(results))
+#'
+residuals.DE <- function(object, ...){
+
+  out =  cbind(object$Y - object$XBeta1, object$Y - object$XBeta2)
+  colnames(out) = names(object$X)
+
+  return(out)
+
+}
+
+#' @title Data from Fair and Jaffee (1972).
+#'
+#' @description
+#' A data set of 126 monthly observations of the housing market from June 1959 to November 1969. The variables are
+#'
+#' \eqn{T}: Time. A continuous values time variable
+#' \eqn{\sum_{i=1}^{t-1}HS_i}{\sum[i=1]^{t-1}HS[i]}: Stock of houses. A sum of housing starts over all previous periods. Assuming initial stock is 0.
+#' \eqn{RM_{t-2}}{RM[t-2]}: Mortgage rate lagged by two months.
+#' \eqn{DF6_{t-1}}{DF6[t-1]}: A six month moving average of $DF_t$. $DF_t$ is the flow of private deposits into savings and loan associations (SLA) and mutual savings banks during period $t$.
+#' \eqn{DHF3_{t-2}}{DHF3[t-2]}: The flow of borrowings by the SLAs from the federal home-loan bank during month t.
+#' \eqn{RM_{t-1}}{RM[t-1]}: Mortgage rate lagged by one month.
+#'
+#' @references Fair, Ray C., and Dwight M. Jaffee. "Methods of estimation for markets in disequilibrium." Econometrica: Journal of the Econometric Society (1972): 497-514.
+"fjdata"
 
